#lang racket
(require racket/mpair)

(define-struct container (hd tl) #:mutable)

(define (make-list-container)
  (make-container empty #f))

(define (add-to-front! lc i)
  (let ([p (mcons i (container-hd lc))]
        [tl (container-tl lc)])
    (unless tl
      (set-container-tl! lc p))
    (set-container-hd! lc p)))

(define (add-to-back! lc i)
  (let ([p (mcons i empty)]
        [tl (container-tl lc)])
    (if tl
        (set-mcdr! tl p)
        (set-container-hd! lc p))
    (set-container-tl! lc p)))

(define (get-list lc)
  (mlist->list (container-hd lc)))

;; ----------------------------------------

;; Tests:
(let ()
  (define (check a b) 
    (unless (equal? a b) (error "test failed")))

  (define lc1 (make-list-container))
  (check '() (get-list lc1))
  (add-to-front! lc1 5) ; start with front
  (check '(5) (get-list lc1))
  (add-to-back! lc1 6)
  (check '(5 6) (get-list lc1))
  (add-to-back! lc1 7)
  (add-to-back! lc1 8)
  (add-to-front! lc1 4)
  (check '(4 5 6 7 8) (get-list lc1))

  (define lc2 (make-list-container))
  (add-to-back! lc2 5) ; start with back
  (check '(5) (get-list lc2))
  (add-to-front! lc2 6)
  (check '(6 5) (get-list lc2)))

;; ----------------------------------------

(define lc (make-list-container))

(for ([i (in-lines)])
  (add-to-back! lc i))

(for-each displayln (get-list lc))
